@@ -16,12 +16,20 @@ import random
 import statistics
 import string
 import time
+import uuid
 import unittest
 
+import pg8000
+from prettytable import PrettyTable
+import pytest
 import pytz
 
 from okera import context, _thrift_api
+from okera._thrift_api import TAuthorizeQueryClient
+from okera._thrift_api import TAuthorizeQueryParams
+from okera._thrift_api import TRequestContext
 from okera._thrift_api import TTypeId, TConfigType
+from okera._thrift_api import TAttribute
 
 DEFAULT_PRESTO_HOST = os.environ['ODAS_TEST_HOST']
 DEFAULT_PRESTO_PORT = os.environ['ODAS_TEST_PORT_PRESTO_COORD_HTTPS']
@@ -30,6 +38,10 @@ DEFAULT_REST_SERVER_PORT = os.environ['ODAS_TEST_PORT_REST_SERVER']
 DEFAULT_ACCESS_PROXY_HOST = os.environ['ODAS_TEST_HOST']
 DEFAULT_ACCESS_PROXY_PORT = os.environ['ODAS_TEST_PORT_ACCESS_PROXY']
 
+PERF_DB_HOST = 'okera-psql-aurora.cluster-cyn8yfvyuugz.us-west-2.rds.amazonaws.com'
+PERF_DB_USER = 'okera_perf_user'
+PERF_DB_DB = 'rpc_perf_db'
+
 TEST_LEVELS = {
     'notests' : 0,
     'smoke' : 1,
@@ -56,6 +68,12 @@ def get_env_var(name, coercer, default):
 def get_bool_env_var(name, default):
     return get_env_var(name, lambda x: str(x).lower() in ['true'], default)
 
+# If true, run the perf benchmarks.
+RUN_PERF = get_env_var('RUN_PERF', bool, False)
+SKIP_PERF_DATA_LOAD = get_env_var('SKIP_PERF_DATA_LOAD', bool, False)
+DEFAULT_ITERS = get_env_var('PERF_ITERATIONS', int, 50)
+QUIET = get_env_var("PERF_QUIET", bool, False)
+
 def test_level_lt(min_level):
     if TEST_LEVEL not in TEST_LEVELS:
         return False
@@ -107,10 +125,12 @@ def get_worker(ctx, host=None, port=None):
 
     return ctx.connect_worker(host=host, port=port)
 
-def get_rest_server_url(scheme='http', host=DEFAULT_REST_SERVER_HOST, port=DEFAULT_REST_SERVER_PORT):
+def get_rest_server_url(scheme='http',
+                        host=DEFAULT_REST_SERVER_HOST, port=DEFAULT_REST_SERVER_PORT):
     return "%s://%s:%s" % (scheme, host, port)
 
-def get_access_proxy_url(scheme='http', host=DEFAULT_ACCESS_PROXY_HOST, port=DEFAULT_ACCESS_PROXY_PORT):
+def get_access_proxy_url(scheme='http',
+                         host=DEFAULT_ACCESS_PROXY_HOST, port=DEFAULT_ACCESS_PROXY_PORT):
     return "%s://%s:%s" % (scheme, host, port)
 
 def measure_latency(iters, fn, msg=None, quiet=False):
@@ -120,7 +140,7 @@ def measure_latency(iters, fn, msg=None, quiet=False):
         fn()
         durations.append((time.time() - start) * 1000)
     durations = sorted(durations)
-    if quiet:
+    if quiet or QUIET:
         return durations
     if msg:
       print("\n%s\n    Iterations: %s" % (msg, iters))
@@ -215,13 +235,114 @@ def list_configs(conn, config_type):
         raise ValueError("Invalid config type.")
     return conn.scan_as_json(table_name)
 
-class SchemaNode():
-    def __init__(self):
-        self.col = None
-        self.children = []
+def read_sql_file(path, **kwargs):
+    sql = open(path, 'r').read()
+    return sql.format(**kwargs)
 
-    @staticmethod
-    def convert(cols, idx, schema):
+class PerfResults():
+    """
+    Utility class for running, storing  and reporing perf results
+    """
+    def __init__(self):
+        self.results = PrettyTable(
+          ['rpc', 'name', 'user', 'mean', '50%', '90%', '99%', 'max', 'iters']
+        )
+
+    def measure(self, fn, msg, user, rpc, iters=DEFAULT_ITERS):
+        """
+        Measure and store the latency of fn().
+          msg: Name  of test cases
+          user: User (typically admin or not, running the function)
+          rpc: Name of RPC/API being used.
+        """
+        if not RUN_PERF:
+            return
+        label = "%s(%s)" % (msg, user)
+        latencies = measure_latency(iters, fn, msg=label)
+        mean = "{:.3f}".format(statistics.mean(latencies))
+        p50 = "{:.3f}".format(latencies[int(len(latencies) * .5)])
+        p90 = "{:.3f}".format(latencies[int(len(latencies) * .9)])
+        p99 = "{:.3f}".format(latencies[int(len(latencies) * .99)])
+        m = "{:.3f}".format(latencies[-1])
+        row = [rpc, msg, user, mean, p50, p90, p99, m, iters]
+        self.results.add_row(row)
+
+    def finalize_results(self):
+        if not  RUN_PERF:
+            return
+        print("\nPerf tests done:")
+        print(self.results)
+
+        # We want some globals that are the same across python modules so we can
+        # correlate the entire run. This can be done by storing a global inside
+        # the pytest object.
+        globals = {}
+        if hasattr(pytest, 'okera_globals'):
+            globals = pytest.okera_globals
+        else:
+            # First time, generate it
+            globals['now'] = datetime.datetime.now()
+            globals['id'] = uuid.uuid1()
+            pytest.okera_globals = globals
+
+        if 'CEREBRO_PERF_DB_PASSWORD' in os.environ:
+            self.persist_results(globals)
+
+    def skip_data_load(self):
+        return SKIP_PERF_DATA_LOAD
+
+    def run_perf(self):
+        return RUN_PERF
+
+    def persist_results(self, globals):
+        print("Persisting results to database.")
+        id = globals['id']
+        now = globals['now']
+        build = -1
+        version = 'latest'
+        owner = ''
+        if 'BUILD_NUMBER' in os.environ:
+            build = int(os.environ['BUILD_NUMBER'])
+        if 'ODAS_DOCKER_VERSION' in os.environ:
+            v = os.environ['ODAS_DOCKER_VERSION']
+            if 'jenkins' not in v:
+                # Jenkins generates latest versions that are unique, ignore
+                version = v
+        if 'OKERA_BENCHMARK_OWNER' in os.environ:
+            owner = os.environ['OKERA_BENCHMARK_OWNER']
+        conn = pg8000.connect(host=PERF_DB_HOST,
+                            user=PERF_DB_USER,
+                            password=os.environ['CEREBRO_PERF_DB_PASSWORD'],
+                            database=PERF_DB_DB,
+                            ssl=True)
+        cur = conn.cursor()
+        for row in self.results:
+            row.border = False
+            row.header = False
+            rpc = row.get_string(fields=['rpc']).strip()
+            name = row.get_string(fields=['name']).strip()
+            user = row.get_string(fields=['user']).strip()
+            mean = row.get_string(fields=['mean']).strip()
+            p50 = row.get_string(fields=['50%']).strip()
+            p90 = row.get_string(fields=['90%']).strip()
+            p99 = row.get_string(fields=['99%']).strip()
+            max = row.get_string(fields=['max']).strip()
+            iters = row.get_string(fields=['iters']).strip()
+
+            values = (id, now, build, version, owner, rpc,
+                      name, user, mean, p50, p90, p99, max, iters)
+            sql = """INSERT INTO perf_results
+                   VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"""
+            cur.execute(sql, values)
+        conn.commit()
+
+class SchemaNode():
+  def __init__(self):
+      self.col = None
+      self.children = []
+
+  @staticmethod
+  def convert(cols, idx, schema):
         node = SchemaNode()
         node.col = cols[idx]
         schema.append(node)
@@ -501,6 +622,13 @@ class TestBase(unittest.TestCase):
         '''
         conn.execute_ddl(ddl % db)
 
+    @staticmethod
+    def _get_t_attribute_obj(namespace, key):
+        attribute = TAttribute()
+        attribute.attribute_namespace = namespace
+        attribute.key = key
+        return attribute
+
     @staticmethod
     def _tag_all_columns(conn, db, dataset, ns, key, create_tag=False):
         if create_tag:
@@ -785,5 +913,29 @@ class TestBase(unittest.TestCase):
                                  json.dumps(expected, indent=2, sort_keys=True))
         return True
 
+    def authorize_query(self, conn, query, user=None, use_tmp_tables=False, client=None,
+                        return_full_result=False, cte=False, token=None):
+        request = TAuthorizeQueryParams()
+        request.sql = query
+        request.requesting_user = user
+        request.use_session_local_tables = use_tmp_tables
+        request.client = client
+        request.cte_rewrite = cte
+        if token:
+            request.ctx = TRequestContext()
+            request.ctx.auth_token = token
+
+        result = conn.service.client.AuthorizeQuery(request)
+        if client == TAuthorizeQueryClient.TEST_ACK:
+            return ''
+        if return_full_result:
+            return ' '.join(result.result_sql.split())
+        self.assertTrue(result.table is None)
+        if result.requires_worker:
+            return None
+        self.assertTrue(result.result_schema is not None or cte)
+        return ' '.join(result.result_sql.split())
+
+
 def random_string(length):
     return ''.join(random.choice(string.ascii_uppercase) for _ in range(length))
