@@ -18,11 +18,11 @@ class AttributesTest(common.TestBase):
         return False
 
     def _verify_attr(self, val, namespace, key, db, tbl, col):
-        self.assertTrue(val.attribute.attribute_namespace == namespace)
-        self.assertTrue(val.attribute.key == key)
-        self.assertTrue(val.database == db)
-        self.assertTrue(val.table == tbl)
-        self.assertTrue(val.column == col)
+        self.assertEqual(val.attribute.attribute_namespace, namespace)
+        self.assertEqual(val.attribute.key, key)
+        self.assertEqual(val.database, db)
+        self.assertEqual(val.table, tbl)
+        self.assertEqual(val.column, col)
 
     def test_basic(self):
         ctx = common.get_test_context()
@@ -44,7 +44,7 @@ class AttributesTest(common.TestBase):
             attributes = conn.list_attributes('abac_test')
             for a in attributes:
                 self.assertTrue(a.id is not None)
-            self.assertTrue(old_len == len(attributes) + 1)
+            self.assertEqual(old_len, len(attributes) + 1)
             self.assertTrue(self._contains_attribute('abac_test', 'v1', attributes),
                             msg=str(attributes))
             self.assertFalse(self._contains_attribute('abac_test', 'v2', attributes),
@@ -100,20 +100,20 @@ class AttributesTest(common.TestBase):
             self.assertTrue('abac_test2' in namespaces)
             self.assertTrue('abac_test3' in namespaces)
             namespaces = conn.list_attribute_namespaces(editable_only=True)
-            self.assertTrue(len(namespaces) == 0)
+            self.assertEqual(len(namespaces), 0)
 
             attributes = conn.list_attributes('abac_test1')
             self.assertTrue(self._contains_attribute('abac_test1', 'v', attributes))
             self.assertFalse(self._contains_attribute('abac_test2', 'v', attributes))
             attributes = conn.list_attributes('abac_test1', editable_only=True)
-            self.assertTrue(len(attributes) == 0)
+            self.assertEqual(len(attributes), 0)
 
             attributes = conn.list_attributes(editable_only=False)
             self.assertTrue(self._contains_attribute('abac_test1', 'v', attributes))
             self.assertTrue(self._contains_attribute('abac_test2', 'v', attributes))
             self.assertTrue(self._contains_attribute('abac_test3', 'v', attributes))
             attributes = conn.list_attributes('', editable_only=True)
-            self.assertTrue(len(attributes) == 0)
+            self.assertEqual(len(attributes), 0)
 
             # Grant this test user all on two namespaces
             ctx.disable_auth()
@@ -178,14 +178,14 @@ class AttributesTest(common.TestBase):
 
             conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % db)
             conn.execute_ddl('CREATE DATABASE %s' % db)
-            self.assertTrue(len(conn.list_datasets(db)) == 0)
+            self.assertEqual(len(conn.list_datasets(db)), 0)
 
             # Create 2 tables and 2 views
             conn.execute_ddl('CREATE TABLE %s.t1(c1 int, c2 int, c3 int)' % db)
             conn.execute_ddl('CREATE TABLE %s.t2(c1 int, c2 int, c3 int)' % db)
             conn.execute_ddl('CREATE VIEW %s.v1 AS SELECT * from %s.t1' % (db, db))
             conn.execute_ddl('CREATE VIEW %s.v2 AS SELECT * from %s.t1' % (db, db))
-            self.assertTrue(len(conn.list_datasets(db)) == 4)
+            self.assertEqual(len(conn.list_datasets(db)), 4)
 
             # Get the attributes on t1, should be empty
             ds = conn.list_datasets(db, name='t1')[0]
@@ -207,7 +207,7 @@ class AttributesTest(common.TestBase):
             ds = conn.list_datasets(db, name='t1')[0]
             attrs = ds.attribute_values
             self.assertTrue(attrs is not None)
-            self.assertTrue(len(attrs) == 1)
+            self.assertEqual(len(attrs), 1)
             self._verify_attr(attrs[0], 'abac_test', 'v1', db, 't1', None)
             self.assertTrue(not self.collect_column_attributes(ds))
 
@@ -222,7 +222,7 @@ class AttributesTest(common.TestBase):
             ds = conn.list_datasets(db, name='t1')[0]
             attrs = ds.attribute_values
             self.assertTrue(attrs is not None)
-            self.assertTrue(len(attrs) == 2)
+            self.assertEqual(len(attrs), 2)
             if attrs[0].attribute.key == 'v1':
                 self._verify_attr(attrs[0], 'abac_test', 'v1', db, 't1', None)
                 self._verify_attr(attrs[1], 'abac_test', 'v2', db, 't1', None)
@@ -341,7 +341,7 @@ class AttributesTest(common.TestBase):
             ds = conn.list_datasets(db, name='test')[0]
             self.assertTrue(ds.attribute_values is None)
             attrs = self.collect_column_attributes(ds)
-            self.assertTrue(len(attrs) == 0)
+            self.assertEqual(len(attrs), 0)
 
             # Execute tag command
             conn.execute_ddl('ALTER TABLE %s.test EXECUTE AUTOTAG' % db)
