@@ -145,7 +145,7 @@ class AuthorizationTest(unittest.TestCase):
             self.assertTrue('okera_sample.*' in objs)
             self.assertTrue('okera_sample.sample' in objs)
             self.assertTrue('okera_sample.sample.record' in objs)
-            self.assertTrue(len(objs) == 3)
+            self.assertEqual(len(objs), 3)
 
             # Revoke on db, should not cascade, this cascade can take a while so sleep
             # first. In the test setup, the refresh is 5 seconds.
@@ -156,7 +156,7 @@ class AuthorizationTest(unittest.TestCase):
             print(objs)
             self.assertTrue('okera_sample.sample' in objs)
             self.assertTrue('okera_sample.sample.record' in objs)
-            self.assertTrue(len(objs) == 2)
+            self.assertEqual(len(objs), 2)
 
             # Revoke on table, should not cascade.
             conn.execute_ddl(
@@ -165,14 +165,14 @@ class AuthorizationTest(unittest.TestCase):
             objs = self.__collect_grant_objects(conn)
             print(objs)
             self.assertTrue('okera_sample.sample.record' in objs)
-            self.assertTrue(len(objs) == 1)
+            self.assertEqual(len(objs), 1)
 
             # Revoke on column.
             conn.execute_ddl(
                 'REVOKE SELECT(record) ON TABLE okera_sample.sample FROM ROLE %s' %\
                 (TEST_ROLE))
             objs = self.__collect_grant_objects(conn)
-            self.assertTrue(len(objs) == 0)
+            self.assertEqual(len(objs), 0)
 
     @staticmethod
     def __is_complex_type(t):
