@@ -31,7 +31,7 @@ TIME_ZONE = get_localzone().zone
 
 DEFAULT_PRESTO_PORT = os.environ['ODAS_TEST_PORT_PRESTO_COORD_HTTPS']
 
-class BasicTest(unittest.TestCase):
+class BasicTest(common.TestBase):
     @staticmethod
     def __table_exists(name, tbls):
         found_table = False
@@ -97,11 +97,11 @@ class BasicTest(unittest.TestCase):
         # Should always pick the first one
         for _ in range(0, 10):
             host, port = ctx._OkeraContext__pick_host(['ghi', 'abc', 'def'], 123, opt)
-            self.assertTrue(host == 'abc')
+            self.assertEqual(host, 'abc')
             self.assertEqual(port, 123)
 
             host, port = ctx._OkeraContext__pick_host(['abc', 'def'], 123, opt2)
-            self.assertTrue(host == 'def')
+            self.assertEqual(host, 'def')
             self.assertEqual(port, 123)
 
         network_hosts = [
@@ -109,11 +109,11 @@ class BasicTest(unittest.TestCase):
             _thrift_api.TNetworkAddress("456", 4)]
         for _ in range(0, 10):
             host, port = ctx._OkeraContext__pick_host(network_hosts, None, opt)
-            self.assertTrue(host == '456')
+            self.assertEqual(host, '456')
             self.assertEqual(port, 4)
 
             host, port = ctx._OkeraContext__pick_host(network_hosts, None, opt3)
-            self.assertTrue(host == '678')
+            self.assertEqual(host, '678')
 
     def test_basic(self):
         ctx = common.get_test_context()
@@ -469,7 +469,7 @@ class BasicTest(unittest.TestCase):
         expected = Decimal(dec_in)
       sql = 'select CAST({} as DECIMAL({},{})) as mycol'.format(dec_in, prec, scale)
       res = planner.scan_as_pandas(sql).values[0][0]
-      self.assertTrue(res == expected)
+      self.assertEqual(res, expected)
 
     def test_decimal_serialization_json(self):
       ctx = common.get_test_context()
@@ -580,7 +580,7 @@ class BasicTest(unittest.TestCase):
             ## verify the new table is query-able
             sql = 'SELECT * FROM customer.' + test_table
             query_result = planner.scan_as_json(sql)
-            self.assertTrue(len(query_result) == 0)
+            self.assertEqual(len(query_result), 0)
 
             ## drop the newly created test table
             drop_table_ddl = "DROP TABLE customer." + test_table
@@ -797,10 +797,10 @@ class BasicTest(unittest.TestCase):
                 datasets['steward_audit_logs'], is_view=True, should_exist=False)
 
     def test_no_sandbox(self):
-      ctx = common.get_test_context()
-      with common.get_planner(ctx) as conn:
-        dbs = conn.list_databases()
-        assert 'okera_sandbox' not in dbs
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            dbs = conn.list_databases()
+            assert 'okera_sandbox' not in dbs
 
     def test_create_unparseable_view(self):
         ctx = common.get_test_context()
@@ -869,7 +869,7 @@ class BasicTest(unittest.TestCase):
 
             # External view with unparseable query should load for describe statements.
             res = planner.execute_ddl("DESCRIBE %s.%s " % (test_db, test_view))
-            self.assertTrue(len(res) == 1)
+            self.assertEqual(len(res), 1)
             res = planner.execute_ddl("DESCRIBE FORMATTED %s.%s " % (test_db, test_view))
             self.assertTrue(test_db in res)
             self.assertTrue("VIRTUAL_VIEW" in res)
@@ -877,5 +877,185 @@ class BasicTest(unittest.TestCase):
 
             planner.execute_ddl("DROP DATABASE IF EXISTS %s CASCADE" % test_db)
 
+    def test_add_partition(self):
+        db = 'test_db_file_partition'
+        add = "ALTER TABLE %s.tbl ADD PARTITION '%s'"
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            self._recreate_test_db(conn, db)
+            create = '''
+            CREATE EXTERNAL TABLE %s.tbl(c1 STRING, c2 STRING)
+            PARTITIONED BY (year int, month int)
+            ROW FORMAT DELIMITED FIELDS TERMINATED BY '|'
+            STORED AS TEXTFILE
+            LOCATION 's3://cerebrodata-test/partition-ymdata'
+            TBLPROPERTIES('okera.auto_partition_recovery.disable'='true')
+            '''
+            conn.execute_ddl(create % db)
+            res = conn.scan_as_json('%s.tbl' % db)
+            self.assertTrue(len(res) == 0)
+
+            conn.execute_ddl(
+                add % (db, 's3://cerebrodata-test/partition-ymdata/year=2016/month=1'))
+            conn.execute_ddl(
+                add % (db, 's3://cerebrodata-test/partition-ymdata/year=2017/month=3/'))
+            res = conn.scan_as_json('%s.tbl' % db)
+            res = sorted(res, key = lambda i: i['c1'])
+            self.assertTrue(len(res) == 2)
+            self.assertEqual(1, res[0]['month'])
+            self.assertEqual(2016, res[0]['year'])
+            self.assertEqual('2016-1', res[0]['c1'])
+            self.assertEqual(3, res[1]['month'])
+            self.assertEqual(2017, res[1]['year'])
+            self.assertEqual('2017-3', res[1]['c1'])
+
+            # This is currently a no-op, as per all ALTER TABLE ADD PARTITION calls
+            conn.execute_ddl(
+                add % (db, 's3://cerebrodata-test/partition-ymdata/year=2016/month=1'))
+
+    def test_path_resolution(self):
+        # This test creates some path resolution aliases by creating conflicts with
+        # the default database.
+        # Tables:
+        #  - name.tbl
+        #  - default.name
+        NAME = 'test_path_resolution'
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            conn.execute_ddl('DROP TABLE IF EXISTS default.%s' % NAME)
+            conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % NAME)
+            conn.execute_ddl('CREATE DATABASE %s' % NAME)
+            conn.execute_ddl("""
+                CREATE TABLE %s.tbl(s string)""" % NAME)
+            res = conn.scan_as_json('select * from %s.tbl' % NAME)
+            self.assertEqual(0, len(res))
+
+            # Create the alias conflict and try again
+            conn.execute_ddl("""
+                CREATE EXTERNAL TABLE default.%s(s string)
+                LOCATION 'dbfs:/foo/bar' """ % NAME)
+
+            res = conn.scan_as_json('select * from %s.tbl' % NAME)
+            self.assertEqual(0, len(res))
+
+    def test_complex_type_keywords(self):
+        DB = "complex_types_keywords_test"
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            self._recreate_test_db(conn, DB)
+            conn.execute_ddl("""
+              create table %s.tbl (
+                  attributes array<struct<`Attribute`:struct<Value:bigint,_id:string, policyproperties:string, comment:string>>>
+              ) stored as parquet
+            """ % DB)
+            conn.execute_ddl("describe %s.tbl" % DB)
+
+    def test_legacy_hud(self):
+        DB = "legacy_hudi_test_db"
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            self._recreate_test_db(conn, DB)
+            conn.execute_ddl(
+        """
+        CREATE EXTERNAL TABLE %s.hudi_nonpartitioned
+        LIKE PARQUET 's3://cerebrodata-test/hudi_parquet/year=2015/month=03/day=16/5f541af5-ca07-4329-ad8c-40fa9b353f35-0_2-103-391_20200210090618.parquet'
+        ROW FORMAT SERDE 'org.apache.hadoop.hive.ql.io.parquet.serde.ParquetHiveSerDe'
+        STORED AS INPUTFORMAT
+          'com.uber.hoodie.hadoop.HoodieInputFormat'
+        OUTPUTFORMAT
+          'org.apache.hadoop.hive.ql.io.parquet.MapredParquetOutputFormat'
+        LOCATION 's3://cerebrodata-test/hudi_parquet/year=2015/month=03/day=16/';
+        """ % DB)
+            desc = conn.execute_ddl("DESCRIBE FORMATTED %s.hudi_nonpartitioned" % DB)
+            self.assertTrue('com.uber.hoodie.hadoop.HoodieInputFormat' in desc)
+            data = conn.scan_as_json('%s.hudi_nonpartitioned' % DB)
+            self.assertEqual(38, len(data))
+
+    def test_dataset_path_rename(self):
+        # Keep DB name the same for the `test_dataset_path_XXX` tests
+        DB = "test_dataset_path_db"
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            ddls = [
+                "DROP DATABASE IF EXISTS %s CASCADE" % (DB),
+                "CREATE DATABASE %s" % (DB),
+                """CREATE EXTERNAL TABLE %s.tbl1 (
+                    N_NATIONKEY SMALLINT,
+                    N_NAME STRING,
+                    N_REGIONKEY SMALLINT,
+                    N_COMMENT STRING)
+                    STORED AS PARQUET
+                    LOCATION 's3://cerebrodata-test/dataset-rename-test'""" % (DB),
+                "CREATE VIEW %s.view1 AS SELECT * FROM %s.tbl1" % (DB, DB),
+            ]
+
+            for ddl in ddls:
+                conn.execute_ddl(ddl)
+
+            res = conn.get_catalog_objects_at(
+                "s3://cerebrodata-test/dataset-rename-test", include_views=True)
+            assert 's3://cerebrodata-test/dataset-rename-test' in res
+            objs = res['s3://cerebrodata-test/dataset-rename-test']
+            assert len(objs) == 2
+            assert ("%s.tbl1" % DB) in objs
+            assert ("%s.view1" % DB) in objs
+
+            conn.execute_ddl('ALTER TABLE %s.tbl1 RENAME TO %s.tbl2' % (DB, DB))
+            conn.execute_ddl('ALTER VIEW %s.view1 RENAME TO %s.view2' % (DB, DB))
+
+            res = conn.get_catalog_objects_at(
+                "s3://cerebrodata-test/dataset-rename-test", include_views=True)
+            assert 's3://cerebrodata-test/dataset-rename-test' in res
+            objs = res['s3://cerebrodata-test/dataset-rename-test']
+            assert len(objs) == 2
+            assert ("%s.tbl2" % DB) in objs
+            assert ("%s.view2" % DB) in objs
+
+    def test_dataset_path_mixed_case(self):
+        # Keep DB name the same for the `test_dataset_path_XXX` tests
+        DB = "test_dataset_path_db"
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            ddls = [
+                "DROP DATABASE IF EXISTS %s CASCADE" % (DB),
+                "CREATE DATABASE %s" % (DB),
+                """CREATE EXTERNAL TABLE %s.TBL1 (
+                    N_NATIONKEY SMALLINT,
+                    N_NAME STRING,
+                    N_REGIONKEY SMALLINT,
+                    N_COMMENT STRING)
+                    STORED AS PARQUET
+                    LOCATION 's3://cerebrodata-test/dataset-rename-test'""" % (DB),
+                """CREATE EXTERNAL TABLE %s.TBL2 (
+                    N_NATIONKEY SMALLINT,
+                    N_NAME STRING,
+                    N_REGIONKEY SMALLINT,
+                    N_COMMENT STRING)
+                    STORED AS PARQUET
+                    LOCATION 's3://cerebrodata-test/dataset-rename-test'""" % (DB),
+                "CREATE VIEW %s.VIEW1 AS SELECT * FROM %s.TBL1" % (DB, DB),
+            ]
+
+            for ddl in ddls:
+                conn.execute_ddl(ddl)
+
+            res = conn.get_catalog_objects_at(
+                "s3://cerebrodata-test/dataset-rename-test", include_views=True)
+            assert 's3://cerebrodata-test/dataset-rename-test' in res
+            objs = res['s3://cerebrodata-test/dataset-rename-test']
+            assert len(objs) == 3
+            assert ("%s.tbl1" % DB) in objs
+            assert ("%s.tbl2" % DB) in objs
+            assert ("%s.view1" % DB) in objs
+
+            conn.execute_ddl('DROP TABLE %s.TbL1' % (DB))
+            conn.execute_ddl('DROP VIEW %s.ViEw1' % (DB))
+            conn.execute_ddl('DROP DATABASE %s CASCADE' % (DB))
+
+            res = conn.get_catalog_objects_at(
+                "s3://cerebrodata-test/dataset-rename-test", include_views=True)
+            assert 's3://cerebrodata-test/dataset-rename-test' not in res
+
+
 if __name__ == "__main__":
     unittest.main()
