@@ -22,6 +22,7 @@ CONFIGS = {
     'catalog_objects': {
         'databases': [],
         'tables': []},
+    'logging_level': "debug",
     'mapped_catalog_attributes':
     [{'name': "Catalog Classification",
       'okera_namespace': "catalog_classification"},
@@ -68,7 +69,7 @@ CREATE EXTERNAL TABLE IF NOT EXISTS %s.%s (
     contactname STRING,
     dealsize STRING, rep STRING
 )
-LOCATION 's3a://okera-datalake/collibra';"""
+LOCATION 's3a://cerebrodata-test/empty';"""
 
 USERS_DDL = """
 CREATE EXTERNAL TABLE IF NOT EXISTS %s.%s (
@@ -77,7 +78,7 @@ CREATE EXTERNAL TABLE IF NOT EXISTS %s.%s (
     gender STRING,
     ccn STRING
 )
-LOCATION 's3a://okera-datalake/collibra';"""
+LOCATION 's3a://cerebrodata-test/empty';"""
 
 DROP_DDL = "DROP DATABASE IF EXISTS %s CASCADE;"
 
