@@ -7,7 +7,6 @@
 
 import unittest
 
-from okera import context
 from okera.tests import pycerebro_test_common as common
 from okera._thrift_api import TConfigType, TRecordServiceException
 
@@ -27,7 +26,8 @@ TEST_CONFIG_KEY2 = "test_configs_key2"
 
 class ConfigsTest(unittest.TestCase):
     def test_basic(self):
-        with context().connect() as conn:
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
             # Get and verify a system config
             configs = list_of_map_to_map(
                 list_configs(conn, TConfigType.SYSTEM_CONFIG), 'key')
@@ -76,8 +76,24 @@ class ConfigsTest(unittest.TestCase):
             self.assertTrue(TEST_CONFIG_KEY1 not in  configs)
             self.assertTrue(TEST_CONFIG_KEY2 not in  configs)
 
+    def test_abac(self):
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            upsert_config(
+                conn, TConfigType.SYSTEM_CONFIG, ['feature.ui.test_feature'], "true")
+            upsert_config(
+                conn, TConfigType.SYSTEM_CONFIG, ['test_feature'], "yes")
+            query = 'SELECT * FROM okera_system.configs WHERE key = "test_feature"'
+            superuser_list = conn.scan_as_json(query)
+            anyuser_list = conn.scan_as_json(query, requesting_user='anyuser')
+
+            self.assertEqual(1, len(superuser_list))
+            self.assertEqual('test_feature', superuser_list[0].get('key'))
+            self.assertEqual([], anyuser_list)
+
     def test_upsert_regex_autotagger(self):
-        with context().connect() as conn:
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
             # Clean up tagging rules from  previous tests
             for c in list_configs(conn, TConfigType.AUTOTAGGER_REGEX):
                 if c['namespace'] == 'test_ns' and c['tag'] == 'rule1':
@@ -95,6 +111,9 @@ class ConfigsTest(unittest.TestCase):
 
             params['tag'] = 'rule1'
             params['expression'] = '*'
+            with self.assertRaises(TRecordServiceException):
+                upsert_config(conn, TConfigType.AUTOTAGGER_REGEX, None, params)
+            params['expression'] = '.*'
             upsert_config(conn, TConfigType.AUTOTAGGER_REGEX, None, params)
 
             rule = None
@@ -104,14 +123,14 @@ class ConfigsTest(unittest.TestCase):
                     break
             self.assertTrue(rule is not None)
             self.assertEqual('test_ns', rule['namespace'])
-            self.assertEqual('*', rule['expression'])
+            self.assertEqual('.*', rule['expression'])
             self.assertEqual('a great regex', rule['description'])
             self.assertTrue(rule['match_column_name'])
             self.assertTrue(rule['match_column_comment'])
             self.assertEqual('great_rule', rule['name'])
 
             # Update the expression
-            params['expression'] = '.*'
+            params['expression'] = 'a.*'
             old_id = rule['id']
             upsert_config(conn, TConfigType.AUTOTAGGER_REGEX, [str(old_id)], params)
 
@@ -122,7 +141,7 @@ class ConfigsTest(unittest.TestCase):
                     break
             self.assertTrue(rule is not None)
             self.assertEqual('test_ns', rule['namespace'])
-            self.assertEqual('.*', rule['expression'])
+            self.assertEqual('a.*', rule['expression'])
             self.assertEqual('great_rule', rule['name'])
             self.assertEqual(old_id, rule['id'])
 
