@@ -177,7 +177,7 @@ class CrawlerIntegrationTest(unittest.TestCase):
                 # This should only find one dataset since flat structures is not the default
                 self._wait_for_crawler(conn, crawler_db, None)
                 datasets = conn.list_dataset_names(crawler_db)
-                self.assertTrue(len(datasets) == 1)
+                self.assertEqual(len(datasets), 1)
                 conn.execute_ddl('DROP TABLE %s.%s' % (crawler_db, 'crawler_flat_structure'))
 
                 # Configure this crawler to accept flat structures
@@ -664,7 +664,7 @@ class CrawlerIntegrationTest(unittest.TestCase):
             conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % crawler_db)
             conn.execute_ddl("CREATE CRAWLER %s SOURCE '%s'" % (crawler, location))
             self._wait_for_crawler(conn, crawler_db, max_sec=10)
-            self.assertTrue(len(conn.list_datasets(crawler_db)) == 0)
+            self.assertEqual(len(conn.list_datasets(crawler_db)), 0)
 
     @unittest.skipIf(common.TEST_LEVEL in ["smoke", "dev"], "Skipping at unit test level")
     @unittest.skipIf(('S3' not in CRAWLER_TEST_MODES),
@@ -859,7 +859,23 @@ class CrawlerIntegrationTest(unittest.TestCase):
             conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % crawler_db)
             conn.execute_ddl("CREATE CRAWLER %s SOURCE '%s'" % (crawler, location))
             self._wait_for_crawler(conn, crawler_db, max_sec=10)
-            self.assertTrue(len(conn.list_datasets(crawler_db)) == 0)
+            self.assertEqual(len(conn.list_datasets(crawler_db)), 0)
+
+    def test_skip_crawl_when_column_name_contain_invalid_chars(self):
+        crawler = 'test_crawl_col_name_with_dot'
+        crawler_db = '_okera_crawler_' + crawler
+        # this bucket contains two datasets, one of which contains
+        # column names with 'dot' in it.
+        location = 's3://cerebro-customers/chase/zd1475'
+
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % crawler_db)
+            conn.execute_ddl("CREATE CRAWLER %s SOURCE '%s'" % (crawler, location))
+            self._wait_for_crawler(conn, crawler_db, max_sec=10)
+            # only one dataset should be found by the crawler
+            # and call should succeed
+            self.assertEqual(len(conn.list_datasets(crawler_db)), 1)
 
     def test_crawler_privileges(self):
         s3_uri = 's3://cerebrodata-test/tpch-nation-integration-test/'
