@@ -89,21 +89,11 @@ class DataRegistrationTest(unittest.TestCase):
         super(DataRegistrationTest, cls).setUpClass()
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
-            conn.manage_data_reg_connection(
-                "DELETE",
-                cls._get_psqltest_data_reg_connection_obj(cls, "test_connection_name"))
-            conn.manage_data_reg_connection(
-                "DELETE",
-                cls._get_psqltest_data_reg_connection_obj(cls, "test_connection_name_2"))
-            conn.manage_data_reg_connection(
-                "DELETE",
-                cls._get_mysqltest_data_reg_connection_obj(cls, "mysql_test_connection"))
-            conn.manage_data_reg_connection(
-                "DELETE",
-                cls._get_mssqltest_data_reg_connection_obj(cls, "mssql_test_connection"))
-            conn.manage_data_reg_connection(
-                "DELETE",
-                cls._get_snowflake_data_reg_connection_obj(cls, "snwflk_test_connection"))
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS test_connection_name')
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS test_connection_name_2')
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS mysql_test_connection')
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS mssql_test_connection')
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS snwflk_test_connection')
 
     def _get_psqltest_data_reg_connection_obj(self, name):
         data_reg_connection = TDataRegConnection()
@@ -335,9 +325,9 @@ class DataRegistrationTest(unittest.TestCase):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
 
-            drc = self._get_psqltest_data_reg_connection_obj("test_connection_name_bc")
             # Clean if already exists
-            drcs = conn.manage_data_reg_connection("DELETE", drc)
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS test_connection_name_bc')
+            drc = self._get_psqltest_data_reg_connection_obj("test_connection_name_bc")
 
             # create test
             drc.username = "awssm://postgres-jdbcpsqltest-username"
@@ -366,7 +356,8 @@ class DataRegistrationTest(unittest.TestCase):
                             "list data registration object connection failed.")
 
             # create another one
-            drc = self._get_psqltest_data_reg_connection_obj("test_connection_name_bc_2")
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS test_connection_name_mysql')
+            drc = self._get_mysqltest_data_reg_connection_obj("test_connection_name_mysql")
             drcs = conn.manage_data_reg_connection("CREATE", drc)
             self.assertTrue(len(drcs.connections) == 1)
 
@@ -377,14 +368,38 @@ class DataRegistrationTest(unittest.TestCase):
             connection_names = [names.name for names in drcs.connections]
             self.assertTrue("test_connection_name_bc" in connection_names,
                             "list data registration object connection failed.")
-            self.assertTrue("test_connection_name_bc_2" in connection_names,
+            self.assertTrue("test_connection_name_mysql" in connection_names,
                             "list data registration object connection failed.")
 
             # list with pattern search works as expected
             drcs = conn.manage_data_reg_connection(
-                "LIST", drc, ["test_connection_name_bc", "test_connection_name_bc_2"])
+                "LIST", drc, ["test_connection_name_bc", "test_connection_name_mysql"])
             self.assertTrue(len(drcs.connections) >= 2)
 
+            # list with jdbc driver search works as expected
+            drcs = conn.manage_data_reg_connection(
+                "LIST", drc, jdbc_driver="mysql")
+            self.assertTrue(len(drcs.connections) >= 1)
+            connection_names = [names.name for names in drcs.connections]
+            self.assertTrue("test_connection_name_mysql" in connection_names,
+                            "list data registration object connection failed.")
+            self.assertTrue("test_connection_name_bc" not in connection_names,
+                            "list data registration object connection failed.")
+
+            # create another mysql connection
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS test_connection_name_2_mysql')
+            drc = self._get_mysqltest_data_reg_connection_obj("test_connection_name_2_mysql")
+            drcs = conn.manage_data_reg_connection("CREATE", drc)
+            self.assertTrue(len(drcs.connections) == 1)
+
+            # list with connection_pattern and jdbc driver search works as expected
+            drcs = conn.manage_data_reg_connection(
+                "LIST", drc, connection_pattern="test_connection_name_bc", jdbc_driver="mysql")
+            self.assertFalse(len(drcs.connections))
+            drcs = conn.manage_data_reg_connection(
+                "LIST", drc, connection_pattern="test_connection_name_2_mysql", jdbc_driver="mysql")
+            self.assertTrue(len(drcs.connections) == 1)
+
             # delete test
             drcs = conn.manage_data_reg_connection("DELETE", drc)
             self.assertTrue(len(drcs.connections) == 0)
@@ -422,7 +437,7 @@ class DataRegistrationTest(unittest.TestCase):
                             "list data registration object connection failed.")
 
             # create another one
-            drc = self._get_psqltest_data_reg_connection_obj("test_connection_name_2")
+            drc = self._get_mysqltest_data_reg_connection_obj("test_connection_name_2")
             drcs = conn.manage_data_reg_connection("CREATE", drc)
             self.assertTrue(len(drcs.connections) == 1)
 
@@ -440,6 +455,31 @@ class DataRegistrationTest(unittest.TestCase):
             drcs = conn.manage_data_reg_connection(
                 "LIST", drc, ["test_connection_name", "test_connection_name_2"])
             self.assertTrue(len(drcs.connections) >= 2)
+
+            # list with jdbc driver search works as expected
+            drcs = conn.manage_data_reg_connection(
+                "LIST", drc, jdbc_driver="mysql")
+            self.assertTrue(len(drcs.connections) >= 1)
+            connection_names = [names.name for names in drcs.connections]
+            self.assertTrue("test_connection_name_2" in connection_names,
+                            "list data registration object connection failed.")
+            self.assertTrue("test_connection_name_bc" not in connection_names,
+                            "list data registration object connection failed.")
+
+            # create another mysql connection
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS test_connection_name_2_mysql')
+            drc = self._get_mysqltest_data_reg_connection_obj("test_connection_name_2_mysql")
+            drcs = conn.manage_data_reg_connection("CREATE", drc)
+            self.assertTrue(len(drcs.connections) == 1)
+
+            # list with connection_pattern and jdbc driver search works as expected
+            drcs = conn.manage_data_reg_connection(
+                "LIST", drc, connection_pattern="test_connection_name_bc", jdbc_driver="mysql")
+            self.assertFalse(len(drcs.connections))
+            drcs = conn.manage_data_reg_connection(
+                "LIST", drc, connection_pattern="test_connection_name_2_mysql", jdbc_driver="mysql")
+            self.assertTrue(len(drcs.connections) == 1)
+
             ## Fixme: This is broken, looks like was always broken
             # self.assertEqual(drcs.connections[0].name,
             #                  "test_connection_name",
@@ -474,8 +514,8 @@ class DataRegistrationTest(unittest.TestCase):
         with common.get_planner(ctx) as conn:
             ctx.disable_auth()
             ddls = [
-                "DROP DATACONNECTION %s" % (MYSQL_CXN),
-                "DROP DATACONNECTION %s" % (SNWFLK_CXN),
+                "DROP DATACONNECTION IF EXISTS %s" % (MYSQL_CXN),
+                "DROP DATACONNECTION IF EXISTS %s" % (SNWFLK_CXN),
             ]
             for ddl in ddls:
                 conn.execute_ddl(ddl)
@@ -595,9 +635,8 @@ class DataRegistrationTest(unittest.TestCase):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
             #MySQL Test
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS mysql_test_connection')
             drc = self._get_mysqltest_data_reg_connection_obj('mysql_test_connection')
-            drcs = conn.manage_data_reg_connection('DELETE', drc)
-            self.assertTrue(len(drcs.connections) == 0)
             drcs = conn.manage_data_reg_connection('CREATE', drc)
             ret_crawler_objects = conn.discover_crawler(drc)
             ret_list_jdbc_datasets = ret_crawler_objects.crawler_discover_datasets[0] \
@@ -608,9 +647,8 @@ class DataRegistrationTest(unittest.TestCase):
                 for x in ret_list_jdbc_datasets))
 
             #Microsoft SQL Server Test
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS mssql_test_connection')
             drc = self._get_mssqltest_data_reg_connection_obj('mssql_test_connection')
-            drcs = conn.manage_data_reg_connection('DELETE', drc)
-            self.assertTrue(len(drcs.connections) == 0)
             drcs = conn.manage_data_reg_connection('CREATE', drc)
             ret_crawler_objects = conn.discover_crawler(drc)
             ret_list_jdbc_datasets = ret_crawler_objects.crawler_discover_datasets[0] \
@@ -627,7 +665,7 @@ class DataRegistrationTest(unittest.TestCase):
             drc = self._get_psqltest_data_reg_connection_obj('test_conn_crawler')
             drc.data_source_path = 's3://cerebro-datasets/jdbc_demo/jdbc_test_psql.conf'
             # delete if it exists then create
-            drcs = conn.manage_data_reg_connection("DELETE", drc)
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS test_conn_crawler')
             drcs = conn.manage_data_reg_connection("CREATE", drc)
             resp = conn.manage_crawler(drc)
             self.assertTrue(resp.status == TCrawlStatus.CRAWLING)
@@ -641,9 +679,8 @@ class DataRegistrationTest(unittest.TestCase):
     def test_discover_crawler_schema(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS mssql_test_connection')
             drc = self._get_mssqltest_data_reg_connection_obj('mssql_test_connection')
-            drcs = conn.manage_data_reg_connection('DELETE', drc)
-            self.assertTrue(len(drcs.connections) == 0)
             drcs = conn.manage_data_reg_connection('CREATE', drc)
             ret_crawler_objects = conn.discover_crawler(drc, 'okera_test')
             ret_list_jdbc_datasets = ret_crawler_objects.crawler_discover_datasets[0] \
@@ -668,9 +705,8 @@ class DataRegistrationTest(unittest.TestCase):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
             #MySQL Test
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS mysql_test_connection')
             drc = self._get_mysqltest_data_reg_connection_obj('mysql_test_connection')
-            drcs = conn.manage_data_reg_connection('DELETE', drc)
-            self.assertTrue(len(drcs.connections) == 0)
             drcs = conn.manage_data_reg_connection('CREATE', drc)
             ret_crawler_objects = conn.discover_crawler(drc, 'jdbc_test')
             ret_list_jdbc_datasets = ret_crawler_objects.crawler_discover_datasets[0] \
@@ -690,9 +726,8 @@ class DataRegistrationTest(unittest.TestCase):
             self.assertTrue(len(drcs.connections) == 0)
 
             #Microsoft SQL Server Test
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS mssql_test_connection')
             drc = self._get_mssqltest_data_reg_connection_obj('mssql_test_connection')
-            drcs = conn.manage_data_reg_connection('DELETE', drc)
-            self.assertTrue(len(drcs.connections) == 0)
             drcs = conn.manage_data_reg_connection('CREATE', drc)
             ret_crawler_objects = conn.discover_crawler(drc, 'okera_test',\
                  'okera_test.dbo')
@@ -741,8 +776,8 @@ class DataRegistrationTest(unittest.TestCase):
                         level, can_show, can_alter, can_drop, can_use = level_test
                         level_name = TAccessPermissionLevel._VALUES_TO_NAMES[level]
                         ddls = [
-                            "DROP DATACONNECTION %s" % (CXN),
-                            "DROP DATACONNECTION %s" % (CXN2),
+                            "DROP DATACONNECTION IF EXISTS %s" % (CXN),
+                            "DROP DATACONNECTION IF EXISTS %s" % (CXN2),
                             "DROP ROLE IF EXISTS %s" % (ROLE),
 
                             # Create the connection
@@ -869,7 +904,7 @@ class DataRegistrationTest(unittest.TestCase):
                                 self.assertTrue('SHOW this data connection' in str(ex_ctx.exception))
 
                         if is_ddl:
-                            stmt = "DROP DATACONNECTION %s" % (CXN)
+                            stmt = "DROP DATACONNECTION IF EXISTS %s" % (CXN)
                             if can_drop:
                                 conn.execute_ddl(stmt)
                             else:
@@ -903,7 +938,7 @@ class DataRegistrationTest(unittest.TestCase):
                 ctx.disable_auth()
 
                 ddls = [
-                    "DROP DATACONNECTION %s" % (CXN),
+                    "DROP DATACONNECTION IF EXISTS %s" % (CXN),
                     "DROP ROLE IF EXISTS %s" % (ROLE),
 
                     # Create the role and grant it to the user
@@ -934,7 +969,7 @@ class DataRegistrationTest(unittest.TestCase):
                             conn.execute_ddl(create_stmt)
 
                             # Since we did create as owner, we can drop it
-                            stmt = "DROP DATACONNECTION %s" % (CXN)
+                            stmt = "DROP DATACONNECTION IF EXISTS %s" % (CXN)
                             conn.execute_ddl(stmt)
                         else:
                             with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
@@ -983,7 +1018,7 @@ class DataRegistrationTest(unittest.TestCase):
             ddls = [
                 "DROP DATABASE IF EXISTS %s cascade" % (DB1),
                 "DROP DATABASE IF EXISTS %s cascade" % (DB2),
-                "DROP DATACONNECTION %s" % (CXN),
+                "DROP DATACONNECTION IF EXISTS %s" % (CXN),
                 "DROP ROLE IF EXISTS %s" % (ROLE),
                 "DROP ROLE IF EXISTS %s" % (ROLE2),
 
@@ -1069,7 +1104,7 @@ class DataRegistrationTest(unittest.TestCase):
             ddls = [
                 "DROP DATABASE IF EXISTS %s cascade" % (DB1),
                 "DROP DATABASE IF EXISTS %s cascade" % (DB2),
-                "DROP DATACONNECTION %s" % (CXN),
+                "DROP DATACONNECTION IF EXISTS %s" % (CXN),
                 "DROP ROLE IF EXISTS %s" % (ROLE),
                 "DROP ROLE IF EXISTS %s" % (ROLE2),
 
@@ -1128,7 +1163,7 @@ class DataRegistrationTest(unittest.TestCase):
 
             ddls = [
                 "DROP DATABASE IF EXISTS %s cascade" % (DB1),
-                "DROP DATACONNECTION %s" % (CXN),
+                "DROP DATACONNECTION IF EXISTS %s" % (CXN),
                 "DROP ROLE IF EXISTS %s" % (ROLE),
                 "DROP ROLE IF EXISTS %s" % (ROLE2),
 
@@ -1194,7 +1229,7 @@ class DataRegistrationTest(unittest.TestCase):
             ddls = [
                 "DROP DATABASE IF EXISTS %s CASCADE" % (DB1),
                 "DROP DATABASE IF EXISTS _okera_crawler_%s CASCADE" % (CXN),
-                "DROP DATACONNECTION %s" % (CXN),
+                "DROP DATACONNECTION IF EXISTS %s" % (CXN),
                 "DROP ROLE IF EXISTS %s" % (ROLE),
                 "DROP ROLE IF EXISTS %s" % (ROLE2),
 
@@ -1249,8 +1284,8 @@ class DataRegistrationTest(unittest.TestCase):
         with common.get_planner(ctx) as conn:
 
             ddls = [
-                "DROP DATACONNECTION %s" % (CXN),
-                "DROP DATACONNECTION %s" % (CXN2),
+                "DROP DATACONNECTION IF EXISTS %s" % (CXN),
+                "DROP DATACONNECTION IF EXISTS %s" % (CXN2),
                 "DROP ROLE IF EXISTS %s" % (ROLE),
                 "DROP ROLE IF EXISTS %s" % (ROLE2),
 
@@ -1410,9 +1445,9 @@ class DataRegistrationTest(unittest.TestCase):
             ctx.disable_auth()
 
             ddls = [
-                "DROP DATACONNECTION %s" % (CXN1),
-                "DROP DATACONNECTION %s" % (CXN2),
-                "DROP DATACONNECTION %s" % (CXN3),
+                "DROP DATACONNECTION IF EXISTS %s" % (CXN1),
+                "DROP DATACONNECTION IF EXISTS %s" % (CXN2),
+                "DROP DATACONNECTION IF EXISTS %s" % (CXN3),
                 "DROP ROLE IF EXISTS %s" % (ROLE1),
                 "DROP ROLE IF EXISTS %s" % (ROLE2),
 
@@ -1519,5 +1554,78 @@ class DataRegistrationTest(unittest.TestCase):
                 conn.manage_data_reg_connection("TEST_EXISTING", drc_cxn3)
             self.assertTrue('The user must have the appropriate permissions' in str(ex_ctx.exception))
 
+    def test_if_exists_clause_when_drop_connection(self):
+        """ 'IF EXISTS' clause can not be specified through API calls.
+            This test just verifies that the overall 'drop/delete connection'
+            functionality works as expected.
+        """
+        MYSQL_CXN = 'test_if_exists_clause_mysql_conn'
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            ctx.disable_auth()
+
+            # DELETE a connection which does not exist, it should throw an error
+            drc = TDataRegConnection()
+            drc.name = MYSQL_CXN
+            with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
+                conn.manage_data_reg_connection('DELETE', drc)
+            self.assertTrue('Connection does not exist: %s' % MYSQL_CXN in str(ex_ctx.exception))
+
+            # create a connection
+            drc_cxn = self._get_mysqltest_data_reg_connection_obj(MYSQL_CXN)
+            conn.manage_data_reg_connection("CREATE", drc_cxn)
+
+            # ensure that the connection is successfully created
+            drcs = conn.manage_data_reg_connection("GET", drc_cxn)
+            self.assertTrue(len(drcs.connections) == 1)
+            self.assertTrue(drcs.connections[0].name == MYSQL_CXN)
+
+            # DELETE the newly created connection
+            drcs = conn.manage_data_reg_connection("DELETE", drc_cxn)
+            self.assertTrue(len(drcs.connections) == 0)
+
+            # get again and it returns empty
+            drcs = conn.manage_data_reg_connection("GET", drc_cxn)
+            self.assertTrue(len(drcs.connections) == 0)
+
+    def test_if_not_exists_clause_when_create_connection(self):
+        """ 'IF NOT EXISTS' clause can not be specified through API calls.
+            This test just verifies that the overall 'create connection'
+            functionality works as expected.
+        """
+        MYSQL_CXN = 'test_if_not_exists_clause_mysql_conn'
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            ctx.disable_auth()
+            # Delete if exists
+            conn.execute_ddl('DROP DATACONNECTION IF EXISTS %s' % MYSQL_CXN)
+
+            # create a connection
+            drc_cxn = self._get_mysqltest_data_reg_connection_obj(MYSQL_CXN)
+            conn.manage_data_reg_connection("CREATE", drc_cxn)
+
+            # ensure that the connection is successfully created
+            drcs = conn.manage_data_reg_connection("GET", drc_cxn)
+            self.assertTrue(len(drcs.connections) == 1)
+            self.assertTrue(drcs.connections[0].name == MYSQL_CXN)
+
+            # Try to create the same connection, it should throw an error.
+            with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
+                conn.manage_data_reg_connection("CREATE", drc_cxn)
+            self.assertTrue('Connection already exists: %s' % MYSQL_CXN in str(ex_ctx.exception))
+
+            # DELETE the connection
+            drcs = conn.manage_data_reg_connection("DELETE", drc_cxn)
+            self.assertTrue(len(drcs.connections) == 0)
+
+            # Re-create the connection and it should work
+            drc_cxn = self._get_mysqltest_data_reg_connection_obj(MYSQL_CXN)
+            conn.manage_data_reg_connection("CREATE", drc_cxn)
+
+            # ensure that the connection is successfully created
+            drcs = conn.manage_data_reg_connection("GET", drc_cxn)
+            self.assertTrue(len(drcs.connections) == 1)
+            self.assertTrue(drcs.connections[0].name == MYSQL_CXN)
+
 if __name__ == "__main__":
     unittest.main()
