@@ -55,26 +55,7 @@ class DenyTest(common.TestBase):
             conn.execute_ddl("CREATE DATABASE %s" % db)
             conn.execute_ddl("CREATE TABLE %s.%s(col1 int, col2 int, col3 int)" % \
                              (db, TEST_TBL))
-
-            conn.execute_ddl('''
-                CREATE EXTERNAL TABLE %s.alltypes(
-                  bool_col BOOLEAN,
-                  tinyint_col TINYINT,
-                  smallint_col SMALLINT,
-                  int_col INT,
-                  bigint_col BIGINT,
-                  float_col FLOAT,
-                  double_col DOUBLE,
-                  string_col STRING,
-                  varchar_col VARCHAR(10),
-                  char_col CHAR(5),
-                  timestamp_col TIMESTAMP,
-                  decimal_col decimal(24, 10))
-                ROW FORMAT DELIMITED FIELDS TERMINATED BY '|'
-                STORED AS TEXTFILE
-                LOCATION 's3://cerebrodata-test/alltypes/'
-                ''' % db)
-
+            common.TestBase._create_all_types(conn, db)
             conn.assign_attribute('deny_test', 'v1', db, 'alltypes', 'int_col')
             conn.assign_attribute('deny_test', 'v1', db, 'alltypes', 'string_col')
             conn.assign_attribute('deny_test', 's1', db, 'alltypes', 'string_col')
@@ -216,6 +197,131 @@ class DenyTest(common.TestBase):
             result = conn.execute_ddl_table_output("SHOW GRANT ROLE %s" % TEST_ROLE)
             print(result)
 
+
+            # Having - drop restrict
+            conn.execute_ddl(
+                "DROP RESTRICT SELECT ON DATABASE %s HAVING ATTRIBUTE %s FROM ROLE %s" % \
+                (TEST_DB, "deny_test.v1", TEST_ROLE))
+
+            # Transform - drop restrict
+            conn.execute_ddl(
+                "DROP RESTRICT SELECT ON DATABASE %s %s FROM ROLE %s" % \
+                (TEST_DB, "TRANSFORM deny_test.v1 WITH mask()", TEST_ROLE))
+
+            # Filter - drop restrict
+            conn.execute_ddl(
+                "DROP RESTRICT SELECT ON DATABASE %s WHERE %s FROM ROLE %s" % \
+                (TEST_DB, "true", TEST_ROLE))
+
+            # Transform and filter - drop restrict
+            conn.execute_ddl(
+                "DROP RESTRICT SELECT ON DATABASE %s %s WHERE %s FROM ROLE %s" % \
+                (TEST_DB, "TRANSFORM deny_test.v1 WITH mask()", "true", TEST_ROLE))
+
+            print("*************** GRANTS after DROP RESTRICT statements: ***********")
+            result = conn.execute_ddl("SHOW GRANT ROLE %s" % TEST_ROLE)
+            print(result)
+            self.assertTrue(not result)
+
+    def test_drop_restrict_ddl(self):
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            self.__cleanup(conn)
+
+            #### Case 1: ####
+            # Try to drop a non-existing restrict privilege, it should throw exception
+            with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
+                conn.execute_ddl(
+                    "DROP RESTRICT SELECT ON DATABASE %s HAVING ATTRIBUTE %s FROM ROLE %s" % \
+                    (TEST_DB, "deny_test.v1", TEST_ROLE))
+            self.assertTrue("CatalogException: Grant WITH EXPRESSION 'deny_test.v1' does not exist."
+                            in str(ex_ctx.exception), msg=str(ex_ctx.exception))
+
+            # Now, create the restrict privilege
+            conn.execute_ddl(
+                "RESTRICT SELECT ON DATABASE %s HAVING ATTRIBUTE %s FROM ROLE %s" % \
+                (TEST_DB, "deny_test.v1", TEST_ROLE))
+            result = conn.execute_ddl("SHOW GRANT ROLE %s" % TEST_ROLE)
+            self.assertEqual(len(result), 1)
+            self.assertEqual(len(result[0]), 16)
+            self.assertTrue(result[0][13])
+
+            # Try to drop newly created restrict privilege, it should be successful
+            conn.execute_ddl(
+                "DROP RESTRICT SELECT ON DATABASE %s HAVING ATTRIBUTE %s FROM ROLE %s" % \
+                (TEST_DB, "deny_test.v1", TEST_ROLE))
+            result = conn.execute_ddl("SHOW GRANT ROLE %s" % TEST_ROLE)
+            self.assertTrue(not result)
+
+
+
+            #### Case 2: ####
+            # Create a new restrict privilege with : Transform and filter
+            conn.execute_ddl(
+                "RESTRICT SELECT ON DATABASE %s %s WHERE %s FROM ROLE %s" % \
+                (TEST_DB, "TRANSFORM deny_test.v1 WITH mask()", "true", TEST_ROLE))
+            result = conn.execute_ddl("SHOW GRANT ROLE %s" % TEST_ROLE)
+            self.assertEqual(len(result), 1)
+            self.assertEqual(len(result[0]), 16)
+            self.assertTrue(result[0][13])
+
+           # Try to drop above restrict with a mismatch in transform function i.e
+           # tokenize() instead of mask(), it should throw an exception
+            with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
+                conn.execute_ddl(
+                    "DROP RESTRICT SELECT ON DATABASE %s %s WHERE %s FROM ROLE %s" % \
+                    (TEST_DB, "TRANSFORM deny_test.v1 WITH tokenize()", "true", TEST_ROLE))
+            self.assertTrue("CatalogException: Grant WITH TRANSFORM 'deny_test.v1:tokenize(__COLUMN__)' WITH FILTER 'TRUE' does not exist."
+                            in str(ex_ctx.exception), msg=str(ex_ctx.exception))
+
+            # Now try to drop it with no mismatch and it should be successful.
+            conn.execute_ddl(
+                "DROP RESTRICT SELECT ON DATABASE %s %s WHERE %s FROM ROLE %s" % \
+                (TEST_DB, "TRANSFORM deny_test.v1 WITH mask()", "true", TEST_ROLE))
+            result = conn.execute_ddl("SHOW GRANT ROLE %s" % TEST_ROLE)
+            self.assertTrue(not result)
+
+            #### Case 3: ####
+            ## To verify that the entry for additional properties (like name, description)
+            ## gets deleted from the kv_store table after DROP RESTRICT command
+
+            # Create a Filter restrict
+            props = " POLICYPROPERTIES('name'='test-drop-restrict', 'description'='policy to test drop restrict.')"
+            conn.execute_ddl(
+                "RESTRICT SELECT ON DATABASE %s WHERE %s FROM ROLE %s %s" % \
+                (TEST_DB, "true", TEST_ROLE, props))
+            result = conn.execute_ddl("SHOW GRANT ROLE %s" % TEST_ROLE)
+            self.assertEqual(len(result), 1)
+            self.assertEqual(len(result[0]), 16)
+            self.assertTrue(result[0][13])
+
+            # Map the OKEAR_DB.KV_STORE table to okera_kv_store table so that we can
+            # run the query against it for verification
+            conn.execute_ddl(
+                """CREATE EXTERNAL TABLE %s.okera_kv_store STORED AS JDBC TBLPROPERTIES(
+                'credentials.file' = '/opt/okera/system_tables.conf',
+                'jdbc.schema.name'='public', 'table'='kv_store')""" % TEST_DB)
+
+            # Verify that an entry for additional properties is created in KV_STORE table.
+            like_clause = "'%test-drop-restrict%'"
+            value_to_verify = """{"name":"test-drop-restrict","description":"policy to test drop restrict."}"""
+            data = conn.scan_as_json(
+                "SELECT * FROM %s.okera_kv_store WHERE value like %s" % (TEST_DB, like_clause))
+            self.assertEqual(len(data), 1)
+            self.assertTrue(value_to_verify in str(data))
+
+            # Now, DROP RESTRICT priv
+            conn.execute_ddl(
+                "DROP RESTRICT SELECT ON DATABASE %s WHERE %s FROM ROLE %s %s" % \
+                (TEST_DB, "true", TEST_ROLE, props))
+            result = conn.execute_ddl("SHOW GRANT ROLE %s" % TEST_ROLE)
+            self.assertTrue(not result)
+
+            # Verify that the entry in the table KV_STORE should also get deleted.
+            data = conn.scan_as_json(
+                "SELECT * FROM %s.okera_kv_store WHERE value like %s" % (TEST_DB, like_clause))
+            self.assertTrue(not data)
+
     def test_grant_props(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -288,7 +394,7 @@ class DenyTest(common.TestBase):
             id4 = priv.props['id']
             del priv.props['id']
             self.assertEqual(priv.props, {'name':'me', 'description':'awesome!'})
-            self.assertTrue(id1 == id4)
+            self.assertEqual(id1, id4)
 
             # Issue an alter
             props = " POLICYPROPERTIES('name'='me3', 'id'='%s')" % id4
@@ -306,7 +412,7 @@ class DenyTest(common.TestBase):
             id5 = priv.props['id']
             del priv.props['id']
             self.assertEqual(priv.props, {'name':'me3'})
-            self.assertTrue(id4 == id5)
+            self.assertEqual(id4, id5)
 
     def test_restrict_filter(self):
         ctx = common.get_test_context()
@@ -320,7 +426,7 @@ class DenyTest(common.TestBase):
                 "RESTRICT SELECT ON DATABASE %s WHERE %s FROM ROLE %s" % \
                 (TEST_DB, "not_a_col = 1", TEST_ROLE))
             data = conn.scan_as_json('%s.alltypes' % TEST_DB)
-            self.assertTrue(len(data) == 2)
+            self.assertEqual(len(data), 2)
 
             # Restricts only, no access granted
             ctx.enable_token_auth(token_str=TEST_USER)
@@ -336,8 +442,8 @@ class DenyTest(common.TestBase):
 
             ctx.enable_token_auth(token_str=TEST_USER)
             data = conn.scan_as_json('%s.alltypes' % TEST_DB)
-            self.assertTrue(len(data) == 1)
-            self.assertTrue(data[0]['int_col'] == 2)
+            self.assertEqual(len(data), 1)
+            self.assertEqual(data[0]['int_col'], 2)
 
     def test_restrict_redact(self):
         ctx = common.get_test_context()
@@ -408,13 +514,17 @@ class DenyTest(common.TestBase):
 
     def test_demo_1(self):
         ctx = common.get_test_context()
+        NOT_VISIBLE_DB = 'not_visible_db'
         with common.get_planner(ctx) as conn:
             self.__cleanup(conn)
+            conn.execute_ddl("CREATE DATABASE IF NOT EXISTS   %s" % NOT_VISIBLE_DB)
 
             # Policy grant with tokenize
             conn.execute_ddl(
                 "GRANT SELECT ON DATABASE %s EXCEPT TRANSFORM %s WITH %s TO ROLE %s" % \
                 (TEST_DB, 'deny_test.s1', 'tokenize()', TEST_ROLE))
+            self.assertTrue(NOT_VISIBLE_DB in conn.list_databases())
+            self.assertTrue(TEST_DB in conn.list_databases())
 
             # Should tokenize
             ctx.enable_token_auth(token_str=TEST_USER)
@@ -423,6 +533,10 @@ class DenyTest(common.TestBase):
             self.assertEqual(data[0]['string_col'], 'yexyi')
             self.assertEqual(data[1]['string_col'], 'gtyjx')
 
+            # Database should not be accessible to this user.
+            self.assertTrue(NOT_VISIBLE_DB not in conn.list_databases())
+            self.assertTrue(TEST_DB in conn.list_databases())
+
             # Safeguard policy with mask should take precedence
             ctx.disable_auth()
             conn.execute_ddl(
@@ -436,6 +550,10 @@ class DenyTest(common.TestBase):
             self.assertEqual(data[0]['string_col'], 'XXXXX')
             self.assertEqual(data[1]['string_col'], 'XXXXX')
 
+            # Safeguard policy should not change it
+            self.assertTrue(NOT_VISIBLE_DB not in conn.list_databases())
+            self.assertTrue(TEST_DB in conn.list_databases())
+
     def test_demo_2(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -475,6 +593,36 @@ class DenyTest(common.TestBase):
             tbls = conn.list_datasets(TEST_DB)
             self.assertEqual(len(tbls), 2)
 
+    @pytest.mark.skip(reason="TDD")
+    def test_filter_ignore_invalid(self):
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            self.__cleanup(conn)
+            props = " POLICYPROPERTIES('row-filter.ignore_missing_columns'='true')"
+            filter = "WHERE not_a_column = 1"
+            conn.execute_ddl(
+                "GRANT SELECT ON CATALOG %s TO ROLE %s %s" % (filter, TEST_ROLE, props))
+
+            ctx.enable_token_auth(token_str=TEST_USER)
+            data = conn.scan_as_json('%s.alltypes' % TEST_DB)
+            print(data)
+
+    @pytest.mark.skip(reason="TDD")
+    def test_filter_invalid(self):
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
+            self.__cleanup(conn)
+            props = " POLICYPROPERTIES('row-filter.ignore_missing_columns'='true')"
+            filter = "WHERE not_a_column = 1"
+            conn.execute_ddl(
+                "GRANT SELECT ON CATALOG %s TO ROLE %s %s" % (filter, TEST_ROLE, props))
+
+            ctx.enable_token_auth(token_str=TEST_USER)
+            with self.assertRaises(_thrift_api.TRecordServiceException) as ex_ctx:
+                conn.scan_as_json('%s.alltypes' % TEST_DB)
+            self.assertTrue('references a column' in str(ex_ctx.exception),
+                            msg=str(ex_ctx.exception))
+
 if __name__ == "__main__":
     unittest.main()
 
