@@ -23,7 +23,7 @@ from okera._thrift_api import (
     TListFilesOp, TListFilesParams
 )
 
-import okera.integration.boto as okera_boto
+import okera_cli.plugin.boto as okera_boto
 
 import botocore
 import boto3
@@ -35,6 +35,7 @@ BUILD_TAG = os.environ.get('BUILD_TAG', random_string(10))
 
 REST_SERVER_URL = common.get_rest_server_url()
 ACCESS_PROXY_URL = common.get_access_proxy_url()
+TOKEN_CONFIG_SOURCE='config'
 
 BOTO_ROOT_BUCKET = "cerebrodata-test"
 BOTO_ROOT_PREFIX = "fs_test/"
@@ -46,13 +47,13 @@ BOTO_FS_USER = 'testbotouser1'
 BOTO_FS_USER_NO_ACCESS = 'testbotouser2'
 BOTO_FS_USER_ROOT = 'root'
 
-USER_SESSION = okera_boto.okera_session(boto3.session.Session(), BOTO_FS_USER, REST_SERVER_URL, ACCESS_PROXY_URL)
+USER_SESSION = okera_boto.okera_session(boto3.session.Session(), BOTO_FS_USER, TOKEN_CONFIG_SOURCE, REST_SERVER_URL, ACCESS_PROXY_URL)
 USER_S3_CLIENT = USER_SESSION.client('s3', verify=False, region_name='us-west-2')
 
-USER_NO_ACCESS_SESSION = okera_boto.okera_session(boto3.session.Session(), BOTO_FS_USER_NO_ACCESS, REST_SERVER_URL, ACCESS_PROXY_URL)
+USER_NO_ACCESS_SESSION = okera_boto.okera_session(boto3.session.Session(), BOTO_FS_USER_NO_ACCESS, TOKEN_CONFIG_SOURCE, REST_SERVER_URL, ACCESS_PROXY_URL)
 USER_NO_ACCESS_S3_CLIENT = USER_NO_ACCESS_SESSION.client('s3', verify=False, region_name='us-west-2')
 
-USER_ROOT_SESSION = okera_boto.okera_session(boto3.session.Session(), BOTO_FS_USER_ROOT, REST_SERVER_URL, ACCESS_PROXY_URL)
+USER_ROOT_SESSION = okera_boto.okera_session(boto3.session.Session(), BOTO_FS_USER_ROOT, TOKEN_CONFIG_SOURCE, REST_SERVER_URL, ACCESS_PROXY_URL)
 USER_ROOT_S3_CLIENT = USER_ROOT_SESSION.client('s3', verify=False, region_name='us-west-2')
 
 def authorize_uri(planner, user, action, uri):
