@@ -6,10 +6,52 @@
 #
 # Tests for Mindsphere API
 #
+#+-----------------+-------------------------------------------+-----------------+---------+---------+---------+---------+---------+-------+
+#|       rpc       |                    name                   |       user      |   mean  |   50%   |   90%   |   99%   |   max   | iters |
+#+-----------------+-------------------------------------------+-----------------+---------+---------+---------+---------+---------+-------+
+#| authorize_query |               abac redaction              | mindsphere_user |  45.039 |  44.190 |  46.937 |  52.385 |  52.385 |  100  |
+#| authorize_query |               abac transform              | mindsphere_user |  81.275 |  81.242 |  83.454 |  96.475 |  96.475 |  100  |
+#| authorize_query |                   admin                   |       None      |  0.275  |  0.268  |  0.299  |  0.571  |  0.571  |  100  |
+#| authorize_query |                  admin100                 |       None      |  1.754  |  1.715  |  1.789  |  5.130  |  5.130  |  100  |
+#| authorize_query |                 admin2000                 |       None      |  31.372 |  30.671 |  33.494 |  40.390 |  41.202 |  300  |
+#| authorize_query |                 admin10000                |       None      | 160.864 | 155.674 | 162.530 | 431.138 | 431.138 |  100  |
+#| authorize_query |                 admin25000                |       None      | 531.898 | 588.496 | 614.410 | 679.952 | 679.952 |   50  |
+#| authorize_query |               all transform               | mindsphere_user |  80.292 |  80.722 |  82.161 |  89.394 |  89.394 |  100  |
+#| authorize_query |              column redaction             | mindsphere_user |  8.552  |  8.441  |  8.637  |  11.164 |  11.164 |  100  |
+#| authorize_query |              dax name filter              | mindsphere_user |  7.742  |  7.509  |  8.177  |  11.110 |  11.110 |  100  |
+#| authorize_query |            filter folding user            | mindsphere_user |  7.819  |  7.724  |  8.046  |  10.470 |  10.470 |  100  |
+#| authorize_query |         filter folding compound 1         | mindsphere_user |  7.814  |  7.754  |  7.885  |  10.286 |  10.286 |  100  |
+#| authorize_query |         filter folding compound 2         | mindsphere_user |  7.865  |  7.726  |  7.895  |  12.941 |  12.941 |  100  |
+#| authorize_query |         filter folding compound 3         | mindsphere_user |  7.992  |  7.858  |  8.068  |  11.502 |  11.502 |  100  |
+#| authorize_query |         filter folding compound 4         | mindsphere_user |  8.017  |  7.927  |  8.112  |  10.495 |  10.495 |  100  |
+#| authorize_query |              hierarchy filter             | mindsphere_user |  7.917  |  7.885  |  8.081  |  10.597 |  10.597 |  100  |
+#| authorize_query |             hierarchy 2 filter            | mindsphere_user |  7.773  |  7.697  |  7.864  |  10.429 |  10.429 |  100  |
+#| authorize_query |             hierarchy 0 filter            | mindsphere_user |  7.678  |  7.609  |  7.750  |  10.039 |  10.039 |  100  |
+#| authorize_query |          hierarchy partial filter         | mindsphere_user |  7.991  |  7.697  |  8.511  |  12.133 |  12.133 |  100  |
+#| authorize_query |           filter: hierarchy.name          | mindsphere_user |  7.471  |  7.424  |  7.516  |  9.897  |  9.897  |  100  |
+#| authorize_query |               country filter              | mindsphere_user |  7.888  |  7.827  |  8.056  |  10.880 |  10.880 |  100  |
+#| authorize_query |            location filter 100            | mindsphere_user |  10.559 |  10.405 |  10.785 |  14.537 |  14.537 |  100  |
+#| authorize_query |              longitude filter             | mindsphere_user |  7.941  |  7.797  |  8.080  |  11.346 |  11.346 |  100  |
+#| authorize_query |                name filter                | mindsphere_user |  7.765  |  7.732  |  7.847  |  10.155 |  10.155 |  100  |
+#| authorize_query |              name filter 100              | mindsphere_user |  11.600 |  11.176 |  12.090 |  17.925 |  17.925 |  100  |
+#| authorize_query |              name filter 2000             | mindsphere_user |  50.589 |  47.711 |  57.690 |  64.125 | 291.683 |  300  |
+#| authorize_query |             name filter 10000             | mindsphere_user | 504.834 | 565.154 | 691.847 | 742.235 | 742.235 |  100  |
+#| authorize_query |        nested collection redaction        | mindsphere_user |  8.439  |  8.348  |  8.471  |  11.436 |  11.436 |  100  |
+#| authorize_query |          nested column redaction          | mindsphere_user |  8.459  |  8.388  |  8.510  |  10.933 |  10.933 |  100  |
+#| authorize_query |              siemens filter 1             | mindsphere_user |  7.972  |  7.902  |  8.076  |  10.183 |  10.183 |  100  |
+#| authorize_query |              siemens filter 2             | mindsphere_user |  8.080  |  7.945  |  8.110  |  16.536 |  16.536 |  100  |
+#| authorize_query |              siemens filter 3             | mindsphere_user |  8.235  |  8.005  |  8.287  |  11.776 |  11.776 |  100  |
+#| authorize_query |            hierarchyPath filter           | mindsphere_user |  7.883  |  7.823  |  7.935  |  10.381 |  10.381 |  100  |
+#| authorize_query |        hierarchyPath filter partial       | mindsphere_user |  7.947  |  7.879  |  8.029  |  10.556 |  10.556 |  100  |
+#| authorize_query | filter: aspects.variable.value = tenantId | mindsphere_user |  7.798  |  7.750  |  7.862  |  10.262 |  10.262 |  100  |
+#| authorize_query |   filter: aspects.variable.value = name   | mindsphere_user |  7.755  |  7.704  |  7.812  |  11.013 |  11.013 |  100  |
+#+-----------------+-------------------------------------------+-----------------+---------+---------+---------+---------+---------+-------+
+
 
 import copy
 import json
 import os
+import pytest
 import unittest
 
 from okera._thrift_api import TAuthorizeQueryParams
@@ -20,8 +62,9 @@ from okera._thrift_api import TRecordServiceException
 
 from okera.tests import pycerebro_test_common as common
 
-MEASURE = common.get_env_var('MEASURE', bool, False)
 ENABLE_OR = common.get_env_var('ENABLE_OR', bool, False)
+PERF = common.PerfResults()
+
 # We don't enable grants into complex types by default. Need to
 # turn on server side with flag.
 # Need to run: start-cdas-container.sh \
@@ -112,6 +155,10 @@ ASSET_RECORD_GERMANY['aspects'][0]['variables'].append({"name": "k", "value": "m
 #print(json.dumps(ASSET_RECORD_GERMANY, indent=2))
 
 class MindsphereApiTest(common.TestBase):
+    @classmethod
+    def tearDownClass(cls):
+        PERF.finalize_results()
+
     @classmethod
     def setUpClass(cls):
         ctx = common.get_test_context()
@@ -159,7 +206,7 @@ class MindsphereApiTest(common.TestBase):
         return ' '.join(result.result_sql.split())
 
     def _authorize_assets(self, label, conn, user, data,
-                          user_filter=None, measure=False, iters=2000,
+                          user_filter=None, measure=False,
                           client = TAuthorizeQueryClient.REST_API):
         input_records = None
         if data is not None:
@@ -171,13 +218,14 @@ class MindsphereApiTest(common.TestBase):
             return self._authorize_query(conn, TEST_DB, 'assets', input_records,
                                          user=user, user_filter=user_filter,
                                          skip_result=True, client=client)
-        if measure or MEASURE:
+        if measure or PERF.run_perf():
             print()
             print('---------------------------- %s --------------------------' % label)
-            common.measure_latency(iters, get)
+            PERF.measure(get, label, user, 'authorize_query')
         return self._authorize_query(conn, TEST_DB, 'assets', input_records, user=user,
                                      user_filter=user_filter, client=client)
 
+    @pytest.mark.perf
     def test_admin(self):
         """ The tests that verify more records spend a lot of time in pytest. """
         ctx = common.get_test_context()
@@ -217,60 +265,27 @@ class MindsphereApiTest(common.TestBase):
             self.assertTrue(filter is None)
             self.assertEqual(records, records_100)
 
-            """
-            ------------------------------ admin2000 ----------------------------
-            Iterations 300
-            Mean 30.05538543065389 ms
-            50%: 29.486656188964844 ms
-            90%: 31.653881072998047 ms
-            95%: 31.9976806640625 ms
-            99%: 38.814544677734375 ms
-            99.5%: 40.86470603942871 ms
-            99.9%: 44.52395439147949 ms
-            """
             records_2000 = []
             for _ in range(0, 2000):
                 records_2000.append(ASSET_FULL_RECORD)
             filter, records, _ = self._authorize_assets(
-                "admin2000", conn, None, records_2000, iters=300)
+                "admin2000", conn, None, records_2000)
             self.assertTrue(filter is None)
             self.assertEqual(records, records_2000)
 
-            """
-            ------------------------------ admin10000 ----------------------------
-            Iterations 100
-            Mean 159.57778453826904 ms
-            50%: 154.77943420410156 ms
-            90%: 160.01248359680176 ms
-            95%: 195.74332237243652 ms
-            99%: 255.62667846679688 ms
-            99.5%: 255.62667846679688 ms
-            99.9%: 255.62667846679688 ms
-            """
             records_10000 = []
             for _ in range(0, 10000):
                 records_10000.append(ASSET_FULL_RECORD)
             filter, records, _ = self._authorize_assets(
-                "admin10000", conn, None, records_10000, iters=100)
+                "admin10000", conn, None, records_10000)
             self.assertTrue(filter is None)
             self.assertEqual(records, records_10000)
 
-            """
-            ------------------------------ admin25000 ----------------------------
-            Iterations 50
-            Mean 465.39061546325684 ms
-            50%: 510.6058120727539 ms
-            90%: 527.1403789520264 ms
-            95%: 528.7942886352539 ms
-            99%: 556.159496307373 ms
-            99.5%: 556.159496307373 ms
-            99.9%: 556.159496307373 ms
-            """
             records_25000 = []
             for _ in range(0, 25000):
                 records_25000.append(ASSET_FULL_RECORD)
             filter, records, _ = self._authorize_assets(
-                "admin25000", conn, None, records_25000, iters=50)
+                "admin25000", conn, None, records_25000)
             self.assertTrue(filter is None)
             self.assertEqual(records, records_25000)
 
@@ -280,6 +295,7 @@ class MindsphereApiTest(common.TestBase):
             self.assertEqual(sql,
                 "SELECT name FROM mindsphere_test.assets")
 
+    @pytest.mark.perf
     def test_name_filter(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -328,42 +344,20 @@ class MindsphereApiTest(common.TestBase):
             self.assertEqual(filter, "name = 'Millenium Falcon'")
             self.assertEqual(records, records_100)
 
-            """
-            ------------------------------ name filter 2000 ----------------------------
-            Iterations 300
-            Mean 37.56293932596842 ms
-            50%: 35.05730628967285 ms
-            90%: 43.69378089904785 ms
-            95%: 47.528982162475586 ms
-            99%: 58.081865310668945 ms
-            99.5%: 62.833309173583984 ms
-            99.9%: 157.52625465393066 ms
-            """
             records_2000 = []
             for _ in range(0, 2000):
                 records_2000.append(ASSET_RECORD_GERMANY)
             filter, records, _ = self._authorize_assets(
-                "name filter 2000", conn, TEST_USER, records_2000, iters=300)
+                "name filter 2000", conn, TEST_USER, records_2000)
             self.assertEqual(filter, "name = 'Millenium Falcon'")
             self.assertTrue(records is None)
 
-            """
-            ------------------------------ name filter 10000 ----------------------------
-            Iterations 100
-            Mean 454.8932361602783 ms
-            50%: 529.1121006011963 ms
-            90%: 583.2486152648926 ms
-            95%: 588.5665416717529 ms
-            99%: 596.2774753570557 ms
-            99.5%: 596.2774753570557 ms
-            99.9%: 596.2774753570557 ms
-            """
             records_10000 = []
             for _ in range(0, 5000):
                 records_10000.append(ASSET_FULL_RECORD)
                 records_10000.append(ASSET_RECORD_GERMANY)
             filter, records, _ = self._authorize_assets(
-                "name filter 10000", conn, TEST_USER, records_10000, iters=100)
+                "name filter 10000", conn, TEST_USER, records_10000)
             self.assertEqual(filter, "name = 'Millenium Falcon'")
             self.assertEqual(len(records), 5000)
 
@@ -376,7 +370,7 @@ class MindsphereApiTest(common.TestBase):
             #    records_10000.append(ASSET_RECORD_GERMANY)
             #    records_10000.append(ASSET_RECORD_GERMANY)
             #filter, records, _ = self._authorize_assets(
-            #    "name filter 25000", conn, TEST_USER, records_25000, iters=30)
+            #    "name filter 25000", conn, TEST_USER, records_25000)
             #self.assertEqual(filter, "name = 'Millenium Falcon'")
             #self.assertEqual(len(records), 5000)
 
@@ -386,6 +380,7 @@ class MindsphereApiTest(common.TestBase):
             self.assertEqual(sql,
                 "SELECT name FROM mindsphere_test.assets WHERE name = 'Millenium Falcon'")
 
+    @pytest.mark.perf
     def test_location_filter(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -487,6 +482,7 @@ class MindsphereApiTest(common.TestBase):
                 "SELECT name FROM mindsphere_test.assets WHERE " +
                 "`location`.longitude > 50")
 
+    @pytest.mark.perf
     def test_hierarchy_filter(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -581,6 +577,7 @@ class MindsphereApiTest(common.TestBase):
                 "SELECT name FROM mindsphere_test.assets " +
                 "WHERE hierarchypath.assetid = 'abcd'")
 
+    @pytest.mark.perf
     def test_variables_filter(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -676,6 +673,7 @@ class MindsphereApiTest(common.TestBase):
                 "SELECT name FROM mindsphere_test.assets WHERE " +
                 "aspects.variables.value = name")
 
+    @pytest.mark.perf
     def test_filter_folding(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -831,6 +829,7 @@ class MindsphereApiTest(common.TestBase):
             self.assertEqual(sql,
                 "SELECT name FROM mindsphere_test.assets")
 
+    @pytest.mark.perf
     def test_siemens_filters(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -889,6 +888,7 @@ class MindsphereApiTest(common.TestBase):
                 '''.split()))
 
     @unittest.skipIf(not ENABLE_OR, "Skipping as this requires flag to enable")
+    @pytest.mark.perf
     def test_multi_grants_or(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -1008,6 +1008,7 @@ class MindsphereApiTest(common.TestBase):
             self.assertEqual(sql,
                 "SELECT name FROM mindsphere_test.assets")
 
+    @pytest.mark.perf
     def test_invalid_filter(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -1045,6 +1046,7 @@ class MindsphereApiTest(common.TestBase):
                     "SELECT name FROM %s.assets" % TEST_DB, user=TEST_USER)
             self.assertTrue('references a column' in str(ex_ctx.exception))
 
+    @pytest.mark.perf
     def test_invalid_collection_filter(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -1078,6 +1080,7 @@ class MindsphereApiTest(common.TestBase):
                 "SELECT name FROM mindsphere_test.assets WHERE " +
                 "aspects.variables.name = variables.value")
 
+    @pytest.mark.perf
     def test_column_redaction(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -1128,15 +1131,10 @@ class MindsphereApiTest(common.TestBase):
             sql = self._authorize_sql(conn,
                 "SELECT * FROM %s.assets" % TEST_DB, user=TEST_USER)
             self.assertEqual(sql,
-                "SELECT name, description, `location`.country as location.country, " +
-                "`location`.region as location.region, " +
-                "`location`.locality as location.locality, " +
-                "`location`.streetaddress as location.streetaddress, " +
-                "`location`.postalcode as location.postalcode, " +
-                "`location`.longitude as location.longitude, " +
-                "`location`.latitude as location.latitude, timezone " +
+                "SELECT name, description, `location` as location, timezone " +
                 "FROM mindsphere_test.assets")
 
+    @pytest.mark.perf
     def test_abac_policy(self):
         """ Test both column pruning and filtering """
         ctx = common.get_test_context()
@@ -1189,13 +1187,17 @@ class MindsphereApiTest(common.TestBase):
                 "SELECT name, description FROM mindsphere_test.assets " +
                 "WHERE name = 'Millenium Falcon'")
 
+    @pytest.mark.perf
     def test_abac_transform(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
             self._recreate_test_role(conn, TEST_ROLE, [TEST_USER])
             conn.delete_attribute('mindsphere_test', 'v')
             conn.create_attribute('mindsphere_test', 'v')
+            conn.delete_attribute('mindsphere_test', 'v2')
+            conn.create_attribute('mindsphere_test', 'v2')
             conn.assign_attribute('mindsphere_test', 'v', TEST_DB, 'assets', 'name')
+            conn.assign_attribute('mindsphere_test', 'v2', TEST_DB, 'assets', 'deleted')
             conn.assign_attribute(
                 'mindsphere_test', 'v', TEST_DB, 'assets', 'description')
             conn.execute_ddl(
@@ -1215,17 +1217,6 @@ class MindsphereApiTest(common.TestBase):
                     "description": "THE SHIP OF HAN SOLO AND CHEWBACCA"
                 }]
 
-            """
-            ---------------------------- abac transform --------------------------
-            Iterations 2000
-            Mean 1.6276593208312988 ms
-            50%: 1.5292167663574219 ms
-            90%: 1.8742084503173828 ms
-            95%: 1.9330978393554688 ms
-            99%: 2.3374557495117188 ms
-            99.5%: 2.4857521057128906 ms
-            99.9%: 6.550788879394531 ms
-            """
             filter, records, res = self._authorize_assets(
                 "abac transform", conn, TEST_USER,
                 [ASSET_FULL_RECORD, ASSET_RECORD_GERMANY])
@@ -1241,6 +1232,32 @@ class MindsphereApiTest(common.TestBase):
                 "SELECT upper(name) as name, upper(description) as description " +
                 "FROM mindsphere_test.assets")
 
+            # Transform on date column
+            conn.execute_ddl(
+                ("GRANT SELECT " +
+                "ON TABLE %s.assets HAVING ATTRIBUTE mindsphere_test.v2 " +
+                "TRANSFORM mindsphere_test.v2 WITH tokenize() " +
+                "TO ROLE %s") %
+                (TEST_DB, TEST_ROLE))
+
+            filter, records, res = self._authorize_assets(
+                "abac transform date", conn, TEST_USER,
+                [ASSET_FULL_RECORD, ASSET_RECORD_GERMANY])
+            print(records)
+            expected = [
+                {
+                    "name": "MILLENIUM FALCON",
+                    "description": "THE SHIP OF HAN SOLO AND CHEWBACCA",
+                    "deleted": "2003-09-17 18:00:29"
+                },
+                {
+                    "name": "MILLENIUM FALCON2",
+                    "description": "THE SHIP OF HAN SOLO AND CHEWBACCA",
+                    "deleted": "2003-09-17 18:00:29"
+                }]
+            self.assertEqual(records, expected)
+
+    @pytest.mark.perf
     def test_all_transform(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -1320,6 +1337,7 @@ class MindsphereApiTest(common.TestBase):
               self.assertEqual(sql,
                   "SELECT tokenize(name) as name FROM mindsphere_test.assets")
 
+    @pytest.mark.perf
     def test_nested_column_redaction(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
@@ -1370,6 +1388,7 @@ class MindsphereApiTest(common.TestBase):
             #    "SELECT upper(name) as name, upper(description) as description " +
             #    "FROM mindsphere_test.assets")
 
+    @pytest.mark.perf
     def test_nested_collection_redaction(self):
         ctx = common.get_test_context()
         with common.get_planner(ctx) as conn:
