@@ -233,7 +233,7 @@ class PrestoTest(common.TestBase):
 
         ctx.enable_token_auth(token_str='some_test_user')
         caught = None
-        expected = "'some_test_user' does not have privileges to access: okera_system.*"
+        expected = "Unknown dataset: okera_system.role_names"
         with common.get_planner(ctx, presto_port=DEFAULT_PRESTO_PORT) as conn:
             try:
                 results = conn.scan_as_json("select * from default.external_view")
@@ -277,7 +277,6 @@ class PrestoTest(common.TestBase):
                     user + ", for sql: " + sql)
 
     def test_complex_type_view_auth(self):
-
         ## verify complex_type where clause auth
         where_qry = """select currency from chase.subscription_view
             where subscriptionkey = 'e23c82e3-10fd-40f3-ba59-5959a82ad3c8'"""
@@ -926,7 +925,8 @@ class PrestoTest(common.TestBase):
 
             # we have some ordering issues when comparing results on some datasets.
             compare_records_returned_only = [
-              'hudi_partitioned'
+              'hudi_partitioned',
+              'lineitem_orc',
             ]
 
             for db in dbs:
@@ -963,7 +963,7 @@ class PrestoTest(common.TestBase):
 
                     if view_ds.lower() in compare_records_returned_only:
                         self.assertEqual(len(presto_view_results),
-                                         len(presto_view_results))
+                                         len(presto_ds_results))
                         self.assertEqual(len(presto_view_results),
                                          len(okera_ds_results))
                         continue
@@ -1048,4 +1048,96 @@ class PrestoTest(common.TestBase):
             assert len(res) == 2
             assert res[1]['row_count'] == 27
 
+    def test_presto_udfs(self):
+        ctx = common.get_test_context()
+        ctx.enable_token_auth(token_str='root')
+        with common.get_planner(ctx, presto_port=DEFAULT_PRESTO_PORT) as conn:
+            res = conn.scan_as_json('select current_database()')
+            assert res[0]['_col0'] == 'okera'
+
+    def test_presto_password(self):
+        query = 'select * from okera_sample.sample'
+        ctx = common.get_test_context()
+        ctx.enable_token_auth(token_str='presto_test_user')
+        with common.get_planner(ctx, presto_port=DEFAULT_PRESTO_PORT) as conn:
+            res = conn.scan_as_json(query, dialect='presto')
+            assert len(res) == 2
+
+            ctx.enable_token_auth(token_str='presto_test_user', presto_password='abc')
+            with pytest.raises(prestodb.exceptions.DatabaseError) as excinfo:
+                conn.scan_as_json(query, dialect='presto')
+            assert "401" in str(excinfo.value)
+
+            ctx.enable_token_auth(token_str='presto_test_user',
+                                  presto_password='presto_test_user')
+            res = conn.scan_as_json(query, dialect='presto')
+            assert len(res) == 2
+
+    def test_plan_error(self):
+        DB = "presto_error_db"
+
+        ctx = common.get_test_context()
+        ctx.enable_token_auth(token_str='root')
+        with common.get_planner(ctx, presto_port=DEFAULT_PRESTO_PORT) as conn:
+            conn.execute_ddl("DROP DATABASE IF EXISTS %s CASCADE" % DB)
+            conn.execute_ddl("CREATE DATABASE %s" % DB)
+            conn.execute_ddl("CREATE TABLE %s.tbl(c1 string)" % DB)
+            conn.execute_ddl("CREATE VIEW %s.vw AS SELECT c1 FROM %s.tbl" % (DB, DB))
+            conn.execute_ddl("ALTER TABLE %s.tbl REPLACE COLUMNS (c2 string)" % DB)
+
+            with pytest.raises(prestodb.exceptions.PrestoQueryError) as excinfo:
+                conn.scan_as_json('select * from %s.vw' % (DB), dialect='presto')
+            assert "Could not resolve column/field reference: 'c1'" in str(excinfo.value)
+
+    def test_keywords_in_column_names(self):
+        ctx = common.get_test_context()
+        with common.get_planner(ctx, presto_port=DEFAULT_PRESTO_PORT) as conn:
+            DB = "presto_test_db"
+            TBL = "test_table"
+
+            ddls = [
+                "DROP DATABASE IF EXISTS %s CASCADE" % DB,
+                "CREATE DATABASE %s" % DB,
+                "CREATE TABLE %s.%s(`database` int, `metadata` struct<`table`: int>)" % (DB, TBL)
+            ]
+
+            for ddl in ddls:
+                conn.execute_ddl(ddl)
+
+            ctx.enable_token_auth(token_str='root')
+            conn.scan_as_json('SELECT * FROM %s.%s' % (DB, TBL), dialect='presto')
+
+    def test_user_delegation(self):
+        ctx = common.get_test_context()
+        with common.get_planner(ctx, presto_port=DEFAULT_PRESTO_PORT) as conn:
+
+            def session_header(username):
+                return {
+                    'X-Presto-User': username
+                }
+
+            ctx.enable_token_auth(token_str='root')
+            res = conn.scan_as_json('SELECT * FROM okera_sample.whoami',
+                              dialect='presto')
+            assert len(res) == 1
+            assert res[0]['user'] == 'root'
+
+            ctx.enable_token_auth(token_str='testuser')
+            res = conn.scan_as_json('SELECT * FROM okera_sample.whoami',
+                              dialect='presto')
+            assert len(res) == 1
+            assert res[0]['user'] == 'testuser'
+
+            ctx.enable_token_auth(token_str='root')
+            conn.scan_as_json('SELECT * FROM okera_sample.whoami',
+                              dialect='presto', presto_headers=session_header('testuser'))
+            assert len(res) == 1
+            assert res[0]['user'] == 'testuser'
+
+            ctx.enable_token_auth(token_str='randomuser')
+            with pytest.raises(prestodb.exceptions.PrestoQueryError) as excinfo:
+                conn.scan_as_json('SELECT * FROM okera_sample.whoami',
+                                  dialect='presto', presto_headers=session_header('testuser'))
+            assert "'randomuser' is not allowed to delegate for user 'testuser'" in str(excinfo.value)
+
     # pylint: enable=protected-access
